/**************************************************************************\
**
**  FilterShell.c
**
**  Sample XTLite bulb to use as a starting point to create import/export
**  filters.
**
**
**  Copyright  1993,1994 Quark Incorporated
**  All Rights Reserved
**
\**************************************************************************/

#include "FilterShell.h"

int32 remaining;					/* num of chars remaining in file buffer */
int32 doffset;						/* offset of char in buffer */
uchar *filebuffer;				/* holds characters */
filtertxtattrib *mytextattr;	/* text attributes */
filterparaattrib *myparaattr;	/* paragraph attributes */
RGBColor thetextcolor;			/* RGB value for text coloring */
static bool8 displaywind;
static int16 maxtabs;
DialogPtr mydlog;

/**************************************************************************\
**
**  mystrcpy
**
**  Build a Pascal string out of the buffer and count that are passed to
**	 writetext.
**
**  Entry: s1 - String to copy to.
**			  s2 - String containing only raw characters.
**			  length - Number of characters in s2.
**
**  Exit:  s1 - Pascal string containing contents of s2.
**
\**************************************************************************/
void mystrcpy(uchar *s1,uchar *s2,int16 length)
{
	
	int16 i;
	for(i = 0;i<length; i++) {
		s1[i+1] = s2[i];
	}
	s1[0] = length;
}


/**************************************************************************\
**
**  pstrcpy
**
**  Copy one Pascal string to another.
**
**  Entry: s1 - String to copy to.
**			  s2 - String to copy from.
**
**  Exit:  s1 - Contains contents of s2.
**
\**************************************************************************/
void pstrcpy(uchar *s1,uchar *s2)
{
	int16 i;

	for(i = (int16)*s2+1; i--;) {
		*s1++ = *s2++;
	}
}


/**************************************************************************\
**
**  pstrconcat
**
**  Add contents of one Pascal string to the end of another.
**
**  Entry: s1 - String to add to.
**			  s2 - String containing additional text to add to s1.
**
**  Exit:  s1 - contains resulting string.
**
\**************************************************************************/
void pstrconcat(uchar *s1,uchar *s2)
{
	int16 len1,len2;

	len1 = *s1;
	len2 = *s2;
	if (len1+len2 > 255) len2 = 255-len1;
	*s1 = len1+len2;
	s1 += len1+1;
	while (len2--) *s1++ = *++s2;
}


/**************************************************************************\
**
**  myFix2str
**
**  Convert a Fixed number to a string.
**
**  Entry: number - the number to convert.
**			  string - the string to build.
**
**  Exit:  string - the string containing the Fixed number.
**
**	 Note:  Fixed numbers are described in Inside Macintosh. They consist
**         of four bytes. The first two bytes (HiWord) contain the integer
**         portion of the number, the last two bytes (LoWord) contain the
**		     fraction portion of the number. The fraction can be found by
**			  multiplying the bottom two bytes by 1/65536. This routine
**			  multiplies that amount by 100 to find the decimal number to be
**   		  place to the right of the decimal point.
**
\**************************************************************************/
void myFix2str(Fixed number,uchar *string)
{
	Str255	tempstr;
	uint32		tempval;
	
	/* Get the integer portion of number */
	tempval = (int32) ((int32)number)>>16;
	NumToString(tempval,string);
	
	/* Place the decimal point */
	pstrconcat(string,"\p.");
	
	/* Get the fraction portion of number */
	tempval = (uint32) (number << 16) >> 16;  /* Creates a int32 containing only bottom
																		 2 bytes */
	tempval = (uint32) (tempval * 100) / 65536; /* Converts bottom 2 bytes into a
																		   decimal number */
	NumToString(tempval,tempstr);
	pstrconcat(string,tempstr);
}
	
/**************************************************************************\
**
**  buildfacestr
**
**  Build a string that contains all of the character attributes currently set.
**
**  Entry: textface - an int16 containing flags for all character attributes.
**			  string - the string to build.
**
**  Exit:  string - contains all of the text attributes.
**
**	 Note:  bold, italic, outline, and shadow are all constants defined in the
**         Mac Toolbox. SUPERSCRIPT, SUBSCRIPT, NONBREAKING, SUPERIOR, 
**         STRIKETHRU, ALLCAPS, SMALLCAPS, WUNDERLINE are constants defined in
**		     the XTLite.h file.
**
\**************************************************************************/
void buildfacestr(int16 textface,uchar *string)
{
	if (textface == 0) {
		pstrcpy(string,"\pPlain");
		return;
	}
	pstrcpy(string,"\p");
	if (textface & bold)
		pstrconcat(string,"\pBold, ");
	if (textface & italic)
		pstrconcat(string,"\pItalic, ");
	if (textface & underline)
		pstrconcat(string,"\pUnderline, ");
	if (textface & outline)
		pstrconcat(string,"\pOutline, ");
	if (textface & shadow)
		pstrconcat(string,"\pShadow, ");
	if (textface & SUPERSCRIPT)
		pstrconcat(string,"\pSuperscript, ");
	if (textface & SUBSCRIPT)
		pstrconcat(string,"\pSubscript, ");
	if (textface & NONBREAKING)
		pstrconcat(string,"\pNonbreaking, ");
	if (textface & SUPERIOR)
		pstrconcat(string,"\pSuperior, ");
	if (textface & STRIKETHRU)
		pstrconcat(string,"\pStrikethru, ");
	if (textface & ALLCAPS)
		pstrconcat(string,"\pAll Caps, ");
	if (textface & SMALLCAPS)
		pstrconcat(string,"\pSmall Caps, ");
	if (textface & WUNDERLINE)
		pstrconcat(string,"\pWord Underline, ");	
	string[0] -= 2; /* Remove last comma and space) */
}
	
/**************************************************************************\
**
**  setupfilter
**
**  Setup the filter information.
**
**  Entry: variables to recieve information.
**
**  Exit:  variables filled.
**
\**************************************************************************/
void setupfilter(OSType *fcreator,OSType *ftype,bool8 *importok,bool8 *exportok,
	Str255 getstr,Str255 savestr,Str255 suffixstr)
{
	*fcreator = OURTEXTCREATOR;			/* set up file creator information */
	*ftype = OURTEXTTYPE;					/* set up file type information */
	*importok = FALSE;
	*exportok = TRUE;							/* we can only export this format */
	GetIndString(savestr,STRINGRESID,EXPORTSTRID);	/* get "Save Text" String */
	suffixstr[0] = 0;
	
	maxtabs = maxnumoftabs();	/* get maximum number of tabs */
}

/**************************************************************************\
**
**  readchar
**
**  Read a character from the file buffer.
**
**  Entry: fnum - the file reference number.
**
**  Exit:  character.
**
\**************************************************************************/
uchar readchar(int16 fnum)
{
	int16 err;

	if (remaining == 0) { 		/* if buffer is empty then read a buffer full */
		remaining = BUFSIZE;
		err = FSRead(fnum,&remaining,filebuffer);
		if (err && !remaining) {
			if (err != eofErr) return (err);
			return (0xFF);
		}
		doffset = 0;
	}
	remaining--;
	return(filebuffer[doffset++]);
}
/**************************************************************************\
**
**  startread
**
**  Initialize variables for start of text import.
**
**  Entry: fnum - the file reference number.
**
**  Exit:  None.
**
\**************************************************************************/
void startread(int16 fnum)
{
}

/**************************************************************************\
**
**  readtext
**
**  Read a section of text for text import.
**
**  Entry: variables to recieve information.
**
**  Exit:  variables filled.
**
\**************************************************************************/
void readtext(int16 fnum,Str255 textbuffer,int32 *count,filtertxtattrib *textattribs,
	filterparaattrib *paraattribs,RGBColor *textcolor)
{
}

/**************************************************************************\
**
**  endread
**
**  De-initialize variables for end of text import.
**
**  Entry: fnum - the file reference number.
**
**  Exit:  None.
**
\**************************************************************************/
void endread(int16 fnum)
{
}

/**************************************************************************\
**
**  startwrite
**
**  Initialize variables for start of text export.
**
**  Entry: fnum - the file reference number.
**
**  Exit:  None.
**
\**************************************************************************/
void startwrite(int16 fnum)
{
	mydlog = GetNewDialog(20000,0,(WindowPtr) -1L);
	displaywind = TRUE;
	ShowWindow(mydlog);	
}

/**************************************************************************\
**
**  writetext
**
**  Write a section of text for text export.
**
**  Entry: variables to recieve information.
**
**  Exit:  variables filled.
**
\**************************************************************************/
void writetext(int16 fnum,Str255 textbuffer,int32 count,filtertxtattrib *textattribs,
	filterparaattrib *paraattribs,RGBColor *textcolor)
{
	char ch;
	int16 i,j;
	int32 size;
	int16 fserr;
	Str255 tempbuffer;
	Str255 tempstr,attribstr;	
	short itype;
	Rect irect;
	Handle ihndl;
	short itemhit;


	/* Display a dialog showing the current text run and character attributes */
	
	if (displaywind) {
		TextFont(geneva);
		TextSize(9);
		
		mystrcpy(tempstr,textbuffer,(int16) count);
		GetDItem(mydlog,3,&itype,&ihndl,&irect);
		SetIText(ihndl,tempstr);
		
		pstrcpy(attribstr,"\pLength: ");
		NumToString(count,tempstr);
		pstrconcat(attribstr,tempstr);
		pstrconcat(attribstr,"\p\r");
		
		pstrconcat(attribstr,"\pFont: ");
		GetFontName(textattribs->font, tempstr);
		pstrconcat(attribstr,tempstr);
		pstrconcat(attribstr,"\p\r");
		
		pstrconcat(attribstr,"\pFace: ");
		buildfacestr(textattribs->face,tempstr);
		pstrconcat(attribstr,tempstr);
		pstrconcat(attribstr,"\p\r");
		
		pstrconcat(attribstr,"\pSize: ");
		myFix2str(textattribs->size,tempstr);
		pstrconcat(attribstr,tempstr);
		pstrconcat(attribstr,"\p\r");
		
		pstrconcat(attribstr,"\pH Scale: ");
		myFix2str(textattribs->hscale,tempstr);
		pstrconcat(attribstr,tempstr);
		pstrconcat(attribstr,"\p\r");
		
		pstrconcat(attribstr,"\pShade: ");
		myFix2str(textattribs->shade,tempstr);
		pstrconcat(attribstr,tempstr);
		pstrconcat(attribstr,"\p\r");
		
		pstrconcat(attribstr,"\pKern: ");
		myFix2str(textattribs->kern,tempstr);
		pstrconcat(attribstr,tempstr);
		pstrconcat(attribstr,"\p\r");
		
		pstrconcat(attribstr,"\pTrack: ");
		myFix2str(textattribs->track,tempstr);
		pstrconcat(attribstr,tempstr);
		pstrconcat(attribstr,"\p\r");
		
		GetDItem(mydlog,4,&itype,&ihndl,&irect);
		SetIText(ihndl,attribstr);
		do {
			ModalDialog(NULL,&itemhit);
			if (itemhit == 2) {
				HideWindow(mydlog);
				displaywind = FALSE;
			}
		} while ( (itemhit != 1) && (itemhit != 2));
		TextFont(0);
		TextSize(0);
	}
	for (j = i = 0; i < count; i++) {
		ch = textbuffer[i];
		if (ch >= ' ' || ch == TABCHAR || ISSPECIAL(ch)) tempbuffer[j++] = ch;
		else if (ISLINECHAR(ch)) tempbuffer[j++] = RETURNCHAR;
	}
	size = j;
	fserr = FSWrite(fnum,&size,tempbuffer);
}

/**************************************************************************\
**
**  endwrite
**
**  De-initialize variables for end of text export.
**
**  Entry: fnum - the file reference number.
**
**  Exit:  None.
**
\**************************************************************************/
void endwrite(int16 fnum)
{
	int32 size;
	int16 fserr;
	
	DisposeDialog(mydlog);
}

/**************************************************************************\
**
**  Remaining code tells XPress we don't want to handle menus or events.
**
\**************************************************************************/
bool8 setupmenu(Str255 menustr)
{
	return (FALSE);
}
void menucall(void) {}

bool8 setupidle(void)
{
	return (FALSE);
}
void idlecall(EventRecord *myevent) {}