/**************************************************************************\
**
**  FilterShell.c
**
**  Sample XTLite bulb to use as a starting point to create import/export
**  filters.
**
**
**  Copyright  1993,1994 Quark Incorporated
**  All Rights Reserved
**
\**************************************************************************/

#include "FilterShell.h"

int32 remaining;					/* num of chars remaining in file buffer */
int32 doffset;						/* offset of char in buffer */
uchar *filebuffer;				/* holds characters */
filtertxtattrib *mytextattr;	/* text attributes */
filterparaattrib *myparaattr;	/* paragraph attributes */
RGBColor thetextcolor;			/* RGB value for text coloring */
static int16 maxtabs;

/**************************************************************************\
**
**  setupfilter
**
**  Setup the filter information.
**
**  Entry: variables to recieve information.
**
**  Exit:  variables filled.
**
\**************************************************************************/
void setupfilter(OSType *fcreator,OSType *ftype,bool8 *importok,bool8 *exportok,
	Str255 getstr,Str255 savestr,Str255 suffixstr)
{
	*fcreator = OURTEXTCREATOR;			/* set up file creator information */
	*ftype = OURTEXTTYPE;					/* set up file type information */
	*importok = *exportok = TRUE;			/* we can import and export this format */
	GetIndString(getstr,STRINGRESID,IMPORTSTRID);	/* get "Get Text" String */
	GetIndString(savestr,STRINGRESID,EXPORTSTRID);	/* get "Save Text" String */
	suffixstr[0] = 0;											/* get ".suffix" String (Optional) */
	
	maxtabs = maxnumoftabs();	/* get maximum number of tabs */
}

/**************************************************************************\
**
**  readchar
**
**  Read a character from the file buffer.
**
**  Entry: fnum - the file reference number.
**
**  Exit:  character.
**
\**************************************************************************/
uchar readchar(int16 fnum)
{
	int16 err;

	if (remaining == 0) { 		/* if buffer is empty then read a buffer full */
		remaining = BUFSIZE;
		err = FSRead(fnum,&remaining,filebuffer);
		if (err && !remaining) {
			if (err != eofErr) return (err);
			return (0xFF);
		}
		doffset = 0;
	}
	remaining--;
	return(filebuffer[doffset++]);
}

/**************************************************************************\
**
**  startread
**
**  Initialize variables for start of text import.
**
**  Entry: fnum - the file reference number.
**
**  Exit:  None.
**
\**************************************************************************/
void startread(int16 fnum)
{
	int16 i;
	
	mytextattr = (filtertxtattribptr)NewPtr(sizeof(filtertxtattrib));
	myparaattr = (filterparaattribptr)NewPtr(sizeof(filterparaattrib)+maxtabs*sizeof(filtertabspec));
	
	mytextattr->font = helvetica;		/* set text to 12 point helvetica */
	mytextattr->size = 12L<<16;		/* point size is in fixed notation, so we << 16 */
	mytextattr->kern = 0L;				/* auto kerning */
	mytextattr->shade = 0x00010000;	/* 100% shade color */
	mytextattr->hscale = 0x00010000;	/* 100% horizontal scaled text */
	mytextattr->track = 0L;
	
	myparaattr->relative = FALSE;		/* not relative leading */
	myparaattr->just = LEFT;			/* left justified text */
	myparaattr->leftindent = 0L;		/* no left indent */
	myparaattr->firstindent = 0L;		/* no first line of paragraph indent */
	myparaattr->rightindent = 0L;		/* no right indent */
	myparaattr->leading = 0L;			/* 0 means auto leading */
	myparaattr->spcbefore = 0L;		/* no space before or after paragraphs */
	myparaattr->spcafter = 0L;

	for (i = 0; i < maxtabs; i++)	/* tabs every .5 inches (all -1L means default) */
		myparaattr->tabs[i].tabindent = -1L;
		
	thetextcolor.red = 0;				/* text color is black */
	thetextcolor.green = 0;
	thetextcolor.blue = 0;
	
	filebuffer = (uchar *)NewPtr(BUFSIZE);
	remaining = doffset = 0;
}

/**************************************************************************\
**
**  readtext
**
**  Read a section of text for text import.
**
**  Entry: variables to recieve information.
**
**  Exit:  variables filled.
**
\**************************************************************************/
void readtext(int16 fnum,Str255 textbuffer,int32 *count,filtertxtattrib *textattribs,
	filterparaattrib *paraattribs,RGBColor *textcolor)
{
	uchar ch;
	int16 i;
			
	i = 0;
	while (TRUE) {
		ch = readchar(fnum);
		/* Handle the characters you read here */
		if (ch >= ' ' || ch == '\t' || ch == '\r')
			textbuffer[i++] = ch;
		if (ch == '\r' || i == 256) break;
		else if (ch == -1) break;
	}
	/* set the text attributes before returning */	
	textattribs->font = mytextattr->font;
	textattribs->face = mytextattr->face;
	textattribs->size = mytextattr->size;
	textattribs->hscale = mytextattr->hscale;
	textattribs->shade = mytextattr->shade;
	textattribs->kern = mytextattr->kern;
	textattribs->track = mytextattr->track;

	paraattribs->relative = myparaattr->relative;
	paraattribs->just = myparaattr->just;
	paraattribs->leftindent = myparaattr->leftindent;
	paraattribs->firstindent = myparaattr->firstindent;
	paraattribs->rightindent = myparaattr->rightindent;
	paraattribs->leading = myparaattr->leading;
	paraattribs->spcbefore = myparaattr->spcbefore;
	paraattribs->spcafter = myparaattr->spcafter;
	BlockMove(paraattribs->tabs,myparaattr->tabs,maxtabs*sizeof(filtertabspec));

	textcolor->red = thetextcolor.red;
	textcolor->green = thetextcolor.green;
	textcolor->blue = thetextcolor.blue;
	
	*count = i;	/* tell QuarkXPress how many characters are in the text buffer */
}

/**************************************************************************\
**
**  endread
**
**  De-initialize variables for end of text import.
**
**  Entry: fnum - the file reference number.
**
**  Exit:  None.
**
\**************************************************************************/
void endread(int16 fnum)
{
	if (filebuffer) DisposPtr((Ptr)filebuffer);
	DisposePtr((Ptr)mytextattr);
	DisposePtr((Ptr)myparaattr);
}

/**************************************************************************\
**
**  startwrite
**
**  Initialize variables for start of text export.
**
**  Entry: fnum - the file reference number.
**
**  Exit:  None.
**
\**************************************************************************/
void startwrite(int16 fnum)
{
}

/**************************************************************************\
**
**  writetext
**
**  Write a section of text for text export.
**
**  Entry: variables to recieve information.
**
**  Exit:  variables filled.
**
\**************************************************************************/
void writetext(int16 fnum,Str255 textbuffer,int32 count,filtertxtattrib *textattribs,
	filterparaattrib *paraattribs,RGBColor *textcolor)
{
	uchar ch;
	int16 i,j;
	int32 size;
	int16 fserr;
	Str255 tempbuffer;
	
	for (j = i = 0; i < count; i++) {
		ch = textbuffer[i];
		/* Remember to filter special characters here */
		/* See the Sample XT Lite bulb for an example */
	}
	size = j;
	fserr = FSWrite(fnum,&size,tempbuffer);
}

/**************************************************************************\
**
**  endwrite
**
**  De-initialize variables for end of text export.
**
**  Entry: fnum - the file reference number.
**
**  Exit:  None.
**
\**************************************************************************/
void endwrite(int16 fnum)
{
	int32 size;
	int16 fserr;
		
}

/**************************************************************************\
**
**  Remaining code tells XPress we don't want to handle menus or events.
**
\**************************************************************************/
bool8 setupmenu(Str255 menustr)
{
	return (FALSE);
}
void menucall(void) {}

bool8 setupidle(void)
{
	return (FALSE);
}
void idlecall(EventRecord *myevent) {}